// File vpTimer16.h for CC1B
#ifndef _VPTIMER16_H_
#define _VPTIMER16_H_
//include other libraries here
#pragma library 1

#ifndef vpTimer16_bankOffset
#define vpTimer16_bankOffset 0x00
#endif

#if (vpTimer16_bankOffset < 0) || (vpTimer16_bankOffset > 11)
#error vpTimer16_bankOffset out of range
#endif

#define vpTimer16(name) \
#ifndef vpTimer16_installed \
#define vpTimer16_installed \
#endif \
#pragma cdata. ##name##_inst = VP_TIMER16,5,0,0,0,0,0 \
#pragma cdata. ##name##_pinconfig \
#pragma cdata[] = 0

shadowDef unsigned long vpTimer16Count @(0x10+vpTimer16_bankOffset); //16bit counter
shadowDef unsigned long vpTimer16Latched @(0x12+vpTimer16_bankOffset); //16bit latch
shadowDef char vpTimer16Flags @(0x14+vpTimer16_bankOffset);
shadowDef bit vpTimer16Clear @vpTimer16Flags.0; //set by mainline code to let the isr code clear the timer
shadowDef bit vpTimer16Latch @vpTimer16Flags.1; //set by mainline code to let the isr code latch the timer
shadowDef bit vpTimer16Hold @vpTimer16Flags.2; //set by mainline code to let the isr code hold the timer

#define VPTIMER16_START 0
#define VPTIMER16_STOP 1

/**
 * Isr code for virtual peripheral timer
 *
 * @param W physical ram bank address
 */
void vpTimer16_isr(char W) {
#pragma update_FSR 0 /* OFF */
  FSR = W;
  if (!vpTimer16Hold) vpTimer16Count++;
  if (vpTimer16Clear) {
    vpTimer16Count = 0;
    vpTimer16Clear = 0;
  }
  if (vpTimer16Latch) {
    vpTimer16Latched = vpTimer16Count;
    vpTimer16Latch = 0;
  }
#pragma update_FSR 1 /* ON */
}

/**
 * Clear timer
 *
 * @param W physical ram bank address
 */
void vpTimer16_clear(char W) {
#pragma update_FSR 0 /* OFF */
  FSR = W;
  vpTimer16Clear = 1; //let isr code clear timer
#pragma update_FSR 1 /* ON */
}

/**
 * Latch timer
 *
 * @param W physical ram bank address
 */
void vpTimer16_latch(char W) {
#pragma update_FSR 0 /* OFF */
  FSR = W;
  vpTimer16Latch = 1; //let isr code update latch
#pragma update_FSR 1 /* ON */
}

/**
 * Hold timer
 *
 * @param W physical ram bank address
 */
void vpTimer16_hold(char W) {
#pragma update_FSR 0 /* OFF */
  FSR = W;
  vpTimer16Hold = 1; //let isr code hold timer
#pragma update_FSR 1 /* ON */
}

/**
 * Run timer
 *
 * @param W physical ram bank address
 */
void vpTimer16_run(char W) {
#pragma update_FSR 0 /* OFF */
  FSR = W;
  vpTimer16Hold = 0; //let isr code run timer
#pragma update_FSR 1 /* ON */
}

/**
 * Get current timer value
 *
 * @param bnk physical ram bank address
 * @return current timer value
 */
unsigned long vpTimer16_timer(char bnk) {
  vpTimer16_latch(bnk);
  W = bnk;
#pragma update_FSR 0 /* OFF */
  FSR = W;
  while (vpTimer16Latch) ; //wait until isr code has updated the latch
#pragma update_FSR 1 /* ON */
  return vpTimer16Latched;
}

/**
 * Get the number of passed ticks since the last call to vpTimer16_mark().
 *
 * @param bnk physical ram bank address
 * @param timer Timer variable that will hold the timer expiration value
 * @return Passed time in ticks
 */
unsigned long vpTimer16_passedTicks(char bnk, unsigned long timer[2]) {
  unsigned long temp = vpTimer16_timer(bnk);
  temp -= timer[VPTIMER16_START];
  return temp;
}

/**
 * Mark timer
 *
 * @param bnk physical ram bank address
 * @param ticks Number of ticks that the timer must advance before it expires
 * @param timer Timer variable that will hold the timer expiration value
 */
void vpTimer16_mark(char bnk, unsigned long ticks, unsigned long timer[2]) {
  unsigned long temp = vpTimer16_timer(bnk);
  timer[VPTIMER16_START] = temp;
  temp += ticks;
  timer[VPTIMER16_STOP] = temp;
}

/**
 * Test for timer timeout
 *
 * @param bnk physical ram bank address
 * @param timer Variable that holds the previously marked timer value (expiration value)
 * @return True if timer expired
 */
bit vpTimer16_timeout(char bnk, unsigned long timer[2]) {
  unsigned long temp = vpTimer16_timer(bnk);
  bit p1 = (temp >= timer[VPTIMER16_STOP]);
  bit p2 = (temp < timer[VPTIMER16_START]);
  if (timer[VPTIMER16_START] < timer[VPTIMER16_STOP]) {
    p1 |= p2;
  }
  else {
    p1 &= p2;
  }
  return p1;
}

#pragma library 0
#endif
// End of file vpTimer16.h
