// File vpobjLib.h for CC1B

/*
	This library requires the following constants to be defined before including the library
	
	#define  _VPLIB_H_VPSLOTS  4     //set number of VP slots (1 to 8)
	#define  _VPLIB_H_VPRAM  0x50  //set bank of first vp ram bank
	#define  _VPLIB_H_vpList  0x10  //set address of char vpList[_VPDEF_H_VPSLOTS]
*/

#ifndef _VPOBJLIB_H_
#define _VPOBJLIB_H_
//include other libraries here
#include <memory.h>
#pragma library 1

#ifndef _VPLIB_H_VPSLOTS
#error _VPLIB_H_VPSLOTS must be defined before including <vplib.h>
#endif
#ifndef _VPLIB_H_VPRAM
#error _VPLIB_H_VPRAM must be defined before including <vplib.h>
#endif
#ifndef _VPLIB_H_vpList
#error _VPLIB_H_vpList must be defined before including <vplib.h>
#endif

//char vpRambank##index##end[16-len] @(_VPLIB_H_VPRAM+(index<<5))+len;
#if defined _CHIP_SX18_ || defined _CHIP_SX20_ || defined _CHIP_SX28_
#define vpobjLib_rambank(index,len) \
char vpRambank##index[len] @(_VPLIB_H_VPRAM+(index<<5)); \
char vpRambank##index##free[16-len] @(_VPLIB_H_VPRAM+(index<<5))+len;
#endif

#if defined _CHIP_SX48_ || defined _CHIP_SX52_
#define vpobjLib_rambank(index,len) \
char vpRambank##index[len] @(_VPLIB_H_VPRAM+(index<<4)); \
char vpRambank##index##free[16-len] @(_VPLIB_H_VPRAM+(index<<4))+len;
#endif

/**
 * vpList array hols vp identifiers
 * highnib of identifier is the entry number from the vp initialization array
 * lownib of identifier is the vp type
 * vpList index number specifies rambank: 0 ->first vp bank, _VPDEF_H_VPSLOTS-1 -> last vp bank
*/

char vpList[_VPLIB_H_VPSLOTS] @_VPLIB_H_vpList;

char vpSlot(char bnk) {
  bnk -= _VPLIB_H_VPRAM;
  bnk >>= 4;
#if defined _CHIP_SX18_ || defined _CHIP_SX20_ || defined _CHIP_SX28_
  bnk >>= 1;
#endif
  return bnk;
}

char vpBank(char vpslot) {
  vpslot <<= 4;
#if defined _CHIP_SX18_ || defined _CHIP_SX20_ || defined _CHIP_SX28_
  vpslot <<= 1;
#endif
  vpslot += _VPLIB_H_VPRAM;
  return vpslot;
}

/**
 * Initialize VP environment
 */
void vpInit() {
  char i;
  for (i=0; i<_VPLIB_H_VPSLOTS; i++) {
    vpList[i] = 0;
  }
}

/**
 * Initialize object at predefined array variable
 * The virtual peripheral initialization data is stored in rom at the passed address
 * The first byte is the length of data that is to be copied
 * to the variable. This byte is followed by 'length' bytes of data.
 * After this are records of 3 bytes, identifying pins. If the first
 * byte of this record is 0, then there are no more pins.
 *
 * @param addr Address of virtual peripheral initialization data
 * @param install 1 to setup pin after intall, 0 to setup pin after uninstall
 */
void pinSetup(long addr, char install) {
  long portpin; char pinconfig;
  portpin.low8 = RomChar(addr); //length
  addr += portpin.low8;
  while (1) {
    addr++;
    portpin.high8 = RomChar(addr);
	if (!portpin.high8) return;
	addr++;
	portpin.low8 = RomChar(addr);
	addr++;
	pinconfig = RomChar(addr); //pinconfig
	if (install) pinconfig = swap(pinconfig);
	if (pinconfig.2) {
	  if (pinconfig.1) setPinInput(portpin);
	  if (pinconfig.0) setPinHigh(portpin);
	  else setPinLow(portpin);
	  if (!pinconfig.1) setPinOutput(portpin);
	}
  }
}

/**
 * Initialize object at predefined array variable
 * The object's initialization data is stored in rom at the passed address
 * The first byte is the length of data that is to be copied
 * to the variable. This byte is followed by 'length' bytes of data.
 *
 * @param var Address of variable (ram array) to be initialized
 * @param addr Address of object initialization data
 */
void objInitialize(char var, long addr) {
  char len = RomChar(addr);
  addr++;
  RomCopy(var,addr,len);
}

/**
 * Install virtual peripheral
 *
 * @param bnk Physical ram bank address
 * @param addr Address of virtual peripheral initialization data
 */
void vpInstall(char bnk, long addr) {
  char vpslot = vpSlot(bnk);
  vpList[vpslot] = 0; //disable current vpslot
  addr++;
  objInitialize(bnk,addr);
  pinSetup(addr,1);
  addr--;
  _MainTemp = RomChar(addr); //type
  vpList[vpslot] = _MainTemp;
}

/**
 * Auto install virtual peripheral
 *
 * @param addr Address of virtual peripheral initialization data
 * @return vpbank where virtual peripheral is installed if succesful, or 0 if not succesful
 */
char vpAutoInstall(long addr) {
  char i;
  for (i=0; i<_VPLIB_H_VPSLOTS; i++) {
    if (vpList[i] == 0) {
	  i = vpBank(i);
	  vpInstall(i,addr);
	  return i;
	}
  }
  return 0;
}

/**
 * Uninstall virtual peripheral
 *
 * @param bnk Physical ram bank address
 * @param addr Address of virtual peripheral initialization data
 */
void vpUninstall(char bnk, long addr) {
  char vpslot = vpSlot(bnk);
  vpList[vpslot] = 0;
  addr++;
  pinSetup(addr,0);
}

#pragma library 0
#endif
// End of file vpobjLib.h
